/*
 * Pins used in this lab
 * Pin  14 on PIC32MX250F128B  -- RB5
 *     TMS/RPB5/USBID/RB5
*/

/* Used chip is PIC32MX250F128B-50I/SP
 *   Will run at 40Mhz since we might have the slower chip some day
 */

#include <xc.h>
#include "configure.h"
#define _SUPPRESS_PLIB_WARNING 1
#include <plib.h>
#include <stdlib.h>
#include <stdint.h>

#include "pitches.h"

#define OUTPUTPIN   5
#define OUTPUTMASK  (1 << OUTPUTPIN)

/*
 * A slightModification of DelayMS found in
 * http://www.look4tech.com/pic32mx220-demo-project-delay-using-32-bit-core-timer/
 */
#define SYS_FREQ 40000000ul
void delayUS(unsigned long usec)
{
    unsigned int tWait;
    tWait=(SYS_FREQ/2000000ul)*usec;
    WriteCoreTimer(0);
    while(ReadCoreTimer() < tWait);
}

/* Data structures from
 *  http://arduino.cc/en/Tutorial/tone
 */

// notes in the melody:
int melody[] = {
  NOTE_C4, NOTE_G3,NOTE_G3, NOTE_A3, NOTE_G3,0, NOTE_B3, NOTE_C4};

// note durations: 4 = quarter note, 8 = eighth note, etc.:
int noteDurations[] = {
  4, 8, 8, 4,4,4,4,4 };

void playTone(int pin, int freqHz, int durationMS) ;

int main(void) {

    ANSELA = 0 ;           // Set all pins for digial
    ANSELB = 0 ;
    CNENA = 0 ;            // Disable change notification
    CNENB = 0 ;
    CNPUA = 0 ;            // Disable weak pullup
    CNPUB = 0 ;
    CNPDA = 0 ;            // Disable weak pulldown
    CNPDB = 0 ;
    ODCA = 0 ;             // Disable open drain
    ODCB = 0 ;
    TRISA = 0 ;            // Set all pins for output
    TRISB = 0 ;
    LATA = 0 ;             // Write 0 to output
    LATB = 0 ;

    // Configuration bits -- 8 / 2 * 20 / 2 or 40 Mhz
    SYSTEMConfigPerformance(SYS_FREQ) ;

    /* Loop modified from
     *  http://arduino.cc/en/Tutorial/tone
     */

    while(1) {
        int thisNote ;
        // iterate over the notes of the melody:
        for (thisNote = 0; thisNote < 8; thisNote++) {
            // to calculate the note duration, take one second
            // divided by the note type.
            //e.g. quarter note = 1000 / 4, eighth note = 1000/8, etc.
            int noteDuration = 1000/noteDurations[thisNote];
            playTone(OUTPUTPIN, melody[thisNote],noteDuration);

            // to distinguish the notes, set a minimum time between them.
            // the note's duration + 30% seems to work well:
            //** int pauseBetweenNotes = noteDuration * 1.30;
            playTone(OUTPUTPIN, 0, noteDuration>>2);
            // stop the tone playing:
            //** noTone(8);
        }
        playTone(OUTPUTPIN, 0, 1000) ;
    }

    return (EXIT_SUCCESS);
}

void playTone(int pin, int frequency, int durationMS) {
    // frequency:  frequency of the tone in cycles per second (Hz)
    //             if frequency is 0, play no tone, that is, take a rest
    // duration:   length of the tone in milliseconds (msec)
    int pinMask = 1<<pin ;
    long durationUS = durationMS * 1000l ;     // now in microSeconds
    if (frequency == 0) {
        delayUS(durationUS) ;
    } else {
        long halfPeriod = 500000l / frequency ;
        long timeSoFar ;
        for (timeSoFar=0 ;timeSoFar<durationUS;timeSoFar += halfPeriod) {
            LATB ^= pinMask ;
            delayUS(halfPeriod) ;
        }
    }
}
